package tracematches;

import java.io.*;

public class ReaderMain {

	static aspect ReaderChecker {
		
		/*
		 * This aspect contains two tracematches.
		 * The first one checks for each reader "r" which is created for an input stream "is", that
		 * neither "r" nor "is" are closed before one reads from "r" or "is".
		 * 
		 * The second tracematch checks the same for a reader that is created for another reader.
		 * 
		 * Unfortunately this does not yet capture the case where we create a reader "r1" on top of a reader
		 * "r2", which was created on top of a stream "is" and then we close "is" and read from "r2".
		 * (c.f. tainted string example for PQL)
		 */
		
		pointcut createReader(InputStream is):
			call(Reader+.new(InputStream+,..)) && args(is,..) ;
			
		tracematch(Reader r, InputStream is) {
			
			sym create after returning(r): createReader(is) && !within(dacapo..*);
			
			sym closeR before: call(* close()) && target(r) && !within(dacapo..*);
			sym closeI before: call(* close()) && target(is) && !within(dacapo..*);
			
			sym readR before: (call(* read(..)) || call(* readLine(..))) && target(r) && !within(dacapo..*);
			sym readI before: (call(* read(..)) || call(* readLine(..))) && target(is) && !within(dacapo..*);
			
			create (closeR|closeI) (readR|readI) {
				if(System.getProperty("TMTEST_ACTIVE")!=null) {
					throw new IllegalStateException("stream or reader closed");			
				}
			}
			
		}

		pointcut createReaderFromReader(Reader r):
			call(Reader+.new(Reader+,..)) && args(r,..) ;

		tracematch(Reader r, Reader inner) {
			
			sym create after returning(r): createReaderFromReader(inner);
			
			sym closeR before: call(* close()) && target(r) && !within(dacapo..*);
			sym closeI before: call(* close()) && target(inner) && !within(dacapo..*);
			
			sym readR before: (call(* read(..)) || call(* readLine(..))) && target(r) && !within(dacapo..*);
			sym readI before: (call(* read(..)) || call(* readLine(..))) && target(inner) && !within(dacapo..*);
			
			create (closeR|closeI) (readR|readI) {
				if(System.getProperty("TMTEST_ACTIVE")!=null) {
					throw new IllegalStateException("stream or reader closed");			
				}
			}
			
		}
	}
}
